@@ -6,6 +6,8 @@ Author: Hubert Tournier
 
 import copy
 import getopt
+import gettext
+import locale
 import logging
 import os
 import re
@@ -14,11 +16,11 @@ import time
 
 import colorama
 
-from .verbs import aux, etre_aux, both_aux
+from .verbs import aux, etre_aux, both_aux, patterns
 from .blank import blank_verb
 
 # Version string used by the what(1) and ident(1) commands:
-ID = "@(#) $Id: conjuguer - conjugaison des verbes Français v0.2.0 (September 11, 2021) by Hubert Tournier $"
+ID = "@(#) $Id: conjuguer - conjugaison des verbes Français v0.4.0 (October 2, 2021) by Hubert Tournier $"
 
 # Default parameters. Can be overcome by environment variables, then command line options
 parameters = {
@@ -46,20 +48,86 @@ DELA = "dict-fr-DELA"
 ABU = "dict-fr-ABU-mots_communs"
 
 
+################################################################################
+def initialize_debugging(program_name):
+    """Debugging set up"""
+    console_log_format = program_name + ": %(levelname)s: %(message)s"
+    logging.basicConfig(format=console_log_format, level=logging.DEBUG)
+    logging.disable(logging.INFO)
+
+
+################################################################################
+def initialize_internationalization(program_name, lang=locale.getdefaultlocale()[0][:2]):
+    """Internationalization set up"""
+    locale_dirs = []
+
+    if os.name == "posix":
+        # local packages override system packages:
+        if "HOME" in os.environ.keys():
+            home = os.environ["HOME"]
+            if os.path.isdir(home + os.sep + ".local/share/locale"):
+                locale_dirs.append(home + os.sep + ".local/share/locale")
+
+        if os.path.isdir("/usr/share/locale"):
+            locale_dirs.append("/usr/share/locale")
+        if os.path.isdir("/usr/local/share/locale"):
+            locale_dirs.append("/usr/local/share/locale")
+    elif os.name == "nt":
+        appdata_path = os.sep + "appdata" + os.sep + "roaming"
+        locale_suffix = os.sep + "python" + os.sep + "share" + os.sep + "locale"
+        if os.environ["APPDATA"]:
+            locale_path = os.environ["APPDATA"] + locale_suffix
+        elif os.environ["HOMEPATH"]:
+            locale_path = os.environ["HOMEPATH"] + appdata_path + locale_suffix
+        elif os.environ["USERPROFILE"]:
+            locale_path = os.environ["USERPROFILE"] + appdata_path + locale_suffix
+        if os.path.isdir(locale_path):
+            locale_dirs.append(locale_path)
+
+        locale_path = sys.base_prefix + os.sep + "share" + os.sep + "locale"
+        if os.path.isdir(locale_path):
+            locale_dirs.append(locale_path)
+
+    for directory in locale_dirs:
+        if gettext.find(program_name, localedir=directory, languages=[lang]) != None:
+            translation = gettext.translation(program_name, localedir=directory, languages=[lang])
+            translation.install()
+            return
+
+    gettext.install(program_name)
+
+
 ################################################################################
 def display_help():
     """Displays usage and help"""
-    print("usage: conjuguer [--debug] [--help|-?] [--version]", file=sys.stderr)
-    print("       [-c|--columns NUMBER] [-d|--dictionary PATH] [-n|--nocolor]", file=sys.stderr)
-    print("       [--] verb [...]", file=sys.stderr)
-    print("  ----------------  -----------------------------------------------------", file=sys.stderr)
-    print("  -c|--columns NUM      Choose number of columns to display between 1, 2 or 4", file=sys.stderr)
-    print("  -d|--dictionary PATH  Select a specific dictionary", file=sys.stderr)
-    print("  -n|--nocolor          Disable color output", file=sys.stderr)
-    print("  --debug               Enable debug mode", file=sys.stderr)
-    print("  --help|-?             Print usage and this help message and exit", file=sys.stderr)
-    print("  --version             Print version and exit", file=sys.stderr)
-    print("  --                    Options processing terminator", file=sys.stderr)
+    print(_("usage: conjuguer [--debug] [--help|-?] [--locale LANG] [--version]"), file=sys.stderr)
+    print(
+        "       "
+        + _("[-c|--columns NUMBER] [-d|--dictionary PATH] [-n|--nocolor]"),
+        file=sys.stderr
+    )
+    print("       " + _("[--] verb [...]"), file=sys.stderr)
+    print(
+        "  " + _("----------------  -----------------------------------------------------"),
+        file=sys.stderr
+    )
+    print(
+        "  " + _("-c|--columns NUM      Choose number of columns to display between 1, 2 or 4"),
+        file=sys.stderr
+    )
+    print("  " + _("-d|--dictionary PATH  Select a specific dictionary"), file=sys.stderr)
+    print("  " + _("-n|--nocolor          Disable color output"), file=sys.stderr)
+    print("  " + _("--debug               Enable debug mode"), file=sys.stderr)
+    print(
+        "  " + _("--help|-?             Print usage and this help message and exit"),
+        file=sys.stderr
+    )
+    print(
+        "  " + _("--locale LANG         Override environment to select another language"),
+        file=sys.stderr
+    )
+    print("  " + _("--version             Print version and exit"), file=sys.stderr)
+    print("  " + _("--                    Options processing terminator"), file=sys.stderr)
     print(file=sys.stderr)
 
 
@@ -93,9 +161,9 @@ def process_environment_variables():
             if os.path.isdir(directory):
                 parameters["DictPath"].append(directory)
             else:
-                logging.warning('DICTPATH directory "%s" not found', directory)
+                logging.warning(_("DICTPATH directory") + ' "%s" ' + _("not found"), directory)
         if len(parameters["DictPath"]) == 0:
-            logging.critical("None of the directories specified in DICTPATH found")
+            logging.critical(_("None of the directories specified in DICTPATH found"))
             sys.exit(1)
     else:
         if os.name == "posix":
@@ -120,7 +188,7 @@ def process_environment_variables():
             if os.path.isdir(pnu_dictpath):
                 parameters["DictPath"].append(pnu_dictpath)
 
-            pnu_dictpath2=sys.executable.replace("python.exe", "share" + os.sep + "dict")
+            pnu_dictpath2 = sys.base_prefix + os.sep + "share" + os.sep + "dict"
             if os.path.isdir(pnu_dictpath2):
                 parameters["DictPath"].append(pnu_dictpath2)
 
@@ -141,13 +209,13 @@ def process_environment_variables():
         if os.path.isfile(os.environ["CONJUGUER_DICT"]):
             parameters["Dictionary path"] = os.environ["CONJUGUER_DICT"]
         else:
-            logging.critical("Dictionary pathname doesn't exist: %s", os.environ["CONJUGUER_DICT"])
+            logging.critical(_("Dictionary pathname doesn't exist") + ": %s", os.environ["CONJUGUER_DICT"])
             sys.exit(1)
 
     if parameters["Dictionary path"]:
         parameters["Dictionary type"] = detect_dictionary_type()
         if parameters["Dictionary type"] not in ("ABU", "DELA"):
-            logging.critical("The selected dictionary doesn't seem to be of ABU or DELA type")
+            logging.critical(_("The selected dictionary doesn't seem to be of ABU or DELA type"))
             sys.exit(1)
 
     logging.debug("process_environment_variables(): parameters:")
@@ -155,7 +223,7 @@ def process_environment_variables():
 
 
 ################################################################################
-def process_command_line():
+def process_command_line(program_name):
     """Process command line options"""
     # pylint: disable=C0103
     global parameters
@@ -169,6 +237,7 @@ def process_command_line():
         "debug",
         "dictionary=",
         "help",
+        "locale=",
         "nocolor",
         "version",
     ]
@@ -178,7 +247,7 @@ def process_command_line():
             sys.argv[1:], character_options, string_options
         )
     except getopt.GetoptError as error:
-        logging.critical("Syntax error: %s", error)
+        logging.critical(_("Syntax error") + ": %s", error)
         display_help()
         sys.exit(1)
 
@@ -188,10 +257,10 @@ def process_command_line():
             try:
                 parameters["Display columns"] = int(argument)
             except ValueError:
-                logging.critical("Option -c/--columns is expecting an integer argument")
+                logging.critical(_("Option -c/--columns is expecting an integer argument"))
                 sys.exit(1)
             if parameters["Display columns"] not in (1, 2, 4):
-                logging.critical("Option -c/--columns is expecting 1, 2 or 4 columns")
+                logging.critical(_("Option -c/--columns is expecting 1, 2 or 4 columns"))
                 sys.exit(1)
 
         elif option == "--debug":
@@ -202,16 +271,21 @@ def process_command_line():
                 parameters["Dictionary path"] = argument
                 parameters["Dictionary type"] = detect_dictionary_type()
                 if parameters["Dictionary type"] not in ("ABU", "DELA"):
-                    logging.critical("The selected dictionary doesn't seem to be of ABU or DELA type")
+                    logging.critical(
+                        _("The selected dictionary doesn't seem to be of ABU or DELA type")
+                    )
                     sys.exit(1)
             else:
-                logging.critical("Option -d/--dictionary is expecting a valid pathname")
+                logging.critical(_("Option -d/--dictionary is expecting a valid pathname"))
                 sys.exit(1)
 
         elif option in ("--help", "-?"):
             display_help()
             sys.exit(0)
 
+        elif option == "--locale":
+            initialize_internationalization(program_name, argument)
+
         elif option in ("-n", "--nocolor"):
             parameters["Color display"] = False
 
@@ -247,7 +321,14 @@ def load_all_verbs_from_dictionary():
                     verbs.append(line)
 
     time_stop = time.time()
-    logging.debug("load_all_verbs_from_dictionary() time: %f / lines: %d", time_stop - time_start, len(verbs))
+    logging.debug(
+        "load_all_verbs_from_dictionary() "
+        + _("time")
+        + ": %f / "
+        + _("lines")
+        + ": %d",
+        time_stop - time_start, len(verbs)
+    )
 
     return verbs
 
@@ -265,11 +346,11 @@ def select_verb_from_verbs(verb, verbs):
             if line.startswith(verb + ",.V"):
                 key = line.split(",")[1]
                 key = re.sub(r":.*", "", key)
-                logging.debug("Key: %s", key)
+                logging.debug(_("Key") + ": %s", key)
                 verb_keys.append(key)
         if len(verb_keys) > 1:
-            logging.warning("More than one key found for %s: %s", verb, " ".join(verb_keys))
-            logging.warning("Only considering the first one")
+            logging.warning(_("More than one key found for") + " %s: %s", verb, " ".join(verb_keys))
+            logging.warning(_("Only considering the first one"))
         if len(verb_keys) == 0:
             return conjugations
 
@@ -278,17 +359,24 @@ def select_verb_from_verbs(verb, verbs):
         re_key = re.escape("," + verb + verb_keys[0])
         for line in verbs:
             if re.search(re_key, line):
-                logging.debug("Line: %s", line)
+                logging.debug(_("Line") + ": %s", line)
                 conjugations.append(line)
 
     elif parameters["Dictionary type"] == "ABU":
         for line in verbs:
             if "	" + verb + "	" in line:
-                logging.debug("Line: %s", line)
+                logging.debug(_("Line") + ": %s", line)
                 conjugations.append(line)
 
     time_stop = time.time()
-    logging.debug("select_verb_from_verbs() time: %f / conjugations: %d", time_stop - time_start, len(conjugations))
+    logging.debug(
+        "select_verb_from_verbs() "
+        + _("time")
+        + ": %f / "
+        + _("conjugations")
+        + ": %d",
+        time_stop - time_start, len(conjugations)
+    )
 
     return conjugations
 
@@ -305,64 +393,89 @@ def fill_verb_from_dela_dictionary_data(verb, conjugations, auxiliary):
     conjugated_verb["Infinitif"]["Présent"] = verb
 
     # We first need to find the "Participe passé" tense for this verb:
-    suffix = ""
+    suffix_s = ""
+    suffix_p = ""
     for line in conjugations:
         for inflection in line.split(":")[1:]:
             if inflection == "Kms":
-                suffix = " " + line.split(",")[0]
-                break
-    if not suffix:
-        logging.warning("Infinitif passé not found for %s", verb)
+                suffix_s = " " + line.split(",")[0]
+            if inflection == "Kmp":
+                suffix_p = " " + line.split(",")[0]
+    if not suffix_s:
+        logging.warning(_("Infinitif passé not found for") + " %s", verb)
     else:
-        conjugated_verb["Infinitif"]["Passé"] = auxiliary + suffix
+        conjugated_verb["Infinitif"]["Passé"] = auxiliary + suffix_s
+    if not suffix_p:
+        suffix_p = suffix_s
 
     for line in conjugations:
         conjugation = line.split(",")[0]
         for inflection in line.split(":")[1:]:
             part = list(inflection)
+            suffix = suffix_s
+            if auxiliary == "être" and len(part) == 3 and part[2] == "p":
+                suffix = suffix_p
+
             if part[0] == "P":
                 conjugated_verb["Indicatif"]["Présent"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Passé composé"][part[2]][part[1]] = aux[auxiliary]["Indicatif"]["Présent"][part[2]][part[1]] + suffix
+                    conjugated_verb["Indicatif"]["Passé composé"][part[2]][part[1]] = \
+                        aux[auxiliary]["Indicatif"]["Présent"][part[2]][part[1]] + suffix
             elif part[0] == "I":
                 conjugated_verb["Indicatif"]["Imparfait"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Plus-que-parfait"][part[2]][part[1]] = aux[auxiliary]["Indicatif"]["Imparfait"][part[2]][part[1]] + suffix
+                    conjugated_verb["Indicatif"]["Plus-que-parfait"][part[2]][part[1]] = \
+                        aux[auxiliary]["Indicatif"]["Imparfait"][part[2]][part[1]] + suffix
             elif part[0] == "J":
                 conjugated_verb["Indicatif"]["Passé simple"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Passé antérieur"][part[2]][part[1]] = aux[auxiliary]["Indicatif"]["Passé simple"][part[2]][part[1]] + suffix
+                    conjugated_verb["Indicatif"]["Passé antérieur"][part[2]][part[1]] = \
+                        aux[auxiliary]["Indicatif"]["Passé simple"][part[2]][part[1]] + suffix
             elif part[0] == "F":
                 conjugated_verb["Indicatif"]["Futur simple"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Futur antérieur"][part[2]][part[1]] = aux[auxiliary]["Indicatif"]["Futur simple"][part[2]][part[1]] + suffix
+                    conjugated_verb["Indicatif"]["Futur antérieur"][part[2]][part[1]] = \
+                        aux[auxiliary]["Indicatif"]["Futur simple"][part[2]][part[1]] + suffix
             elif part[0] == "C":
                 conjugated_verb["Conditionnel"]["Présent"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Conditionnel"]["Passé"][part[2]][part[1]] = aux[auxiliary]["Conditionnel"]["Présent"][part[2]][part[1]] + suffix
+                    conjugated_verb["Conditionnel"]["Passé"][part[2]][part[1]] = \
+                        aux[auxiliary]["Conditionnel"]["Présent"][part[2]][part[1]] + suffix
             elif part[0] == "S":
                 conjugated_verb["Subjonctif"]["Présent"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Subjonctif"]["Passé"][part[2]][part[1]] = aux[auxiliary]["Subjonctif"]["Présent"][part[2]][part[1]] + suffix
+                    conjugated_verb["Subjonctif"]["Passé"][part[2]][part[1]] = \
+                        aux[auxiliary]["Subjonctif"]["Présent"][part[2]][part[1]] + suffix
             elif part[0] == "T":
                 conjugated_verb["Subjonctif"]["Imparfait"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Subjonctif"]["Plus-que-parfait"][part[2]][part[1]] = aux[auxiliary]["Subjonctif"]["Imparfait"][part[2]][part[1]] + suffix
+                    conjugated_verb["Subjonctif"]["Plus-que-parfait"][part[2]][part[1]] = \
+                        aux[auxiliary]["Subjonctif"]["Imparfait"][part[2]][part[1]] + suffix
             elif part[0] == "Y":
                 conjugated_verb["Impératif"]["Présent"][part[2]][part[1]] = conjugation
                 if suffix:
-                    conjugated_verb["Impératif"]["Passé"][part[2]][part[1]] = aux[auxiliary]["Impératif"]["Présent"][part[2]][part[1]] + suffix
+                    conjugated_verb["Impératif"]["Passé"][part[2]][part[1]] = \
+                        aux[auxiliary]["Impératif"]["Présent"][part[2]][part[1]] + suffix
             elif part[0] == "G":
                 conjugated_verb["Participe"]["Présent"] = conjugation
                 conjugated_verb["Gérondif"]["Présent"] = conjugation
                 if suffix:
-                    conjugated_verb["Participe"]["Passé"]["a"] = aux[auxiliary]["Participe"]["Présent"] + suffix
-                    conjugated_verb["Gérondif"]["Passé"] = conjugated_verb["Participe"]["Passé"]["a"]
+                    conjugated_verb["Participe"]["Passé"]["a"] = \
+                        aux[auxiliary]["Participe"]["Présent"] + suffix
+                    conjugated_verb["Gérondif"]["Passé"] = \
+                        conjugated_verb["Participe"]["Passé"]["a"]
             elif part[0] == "K":
                 conjugated_verb["Participe"]["Passé"][part[2]][part[1]] = conjugation
 
     time_stop = time.time()
-    logging.debug("fill_verb_from_dela_dictionary_data() time: %f / conjugations: %d", time_stop - time_start, len(conjugations))
+    logging.debug(
+        "fill_verb_from_dela_dictionary_data() "
+        + _("time")
+        + ": %f / "
+        + _("conjugations")
+        + ": %d",
+        time_stop - time_start, len(conjugations)
+    )
 
     return conjugated_verb
 
@@ -379,16 +492,20 @@ def fill_verb_from_abu_dictionary_data(verb, conjugations, auxiliary):
     conjugated_verb["Infinitif"]["Présent"] = verb
 
     # We first need to find the "Participe passé" tense for this verb:
-    suffix = ""
+    suffix_s = ""
+    suffix_p = ""
     for line in conjugations:
         for inflection in line.split(":")[1:]:
             if inflection == "PPas+Mas+SG":
-                suffix = " " + line.split("	")[0]
-                break
-    if not suffix:
-        logging.warning("Infinitif passé not found for %s", verb)
+                suffix_s = " " + line.split("	")[0]
+            if inflection == "PPas+Mas+PL":
+                suffix_p = " " + line.split("	")[0]
+    if not suffix_s:
+        logging.warning(_("Infinitif passé not found for") + " %s", verb)
     else:
-        conjugated_verb["Infinitif"]["Passé"] = auxiliary + suffix
+        conjugated_verb["Infinitif"]["Passé"] = auxiliary + suffix_s
+    if not suffix_p:
+        suffix_p = suffix_s
 
     for line in conjugations:
         conjugation = line.split("	")[0]
@@ -418,50 +535,70 @@ def fill_verb_from_abu_dictionary_data(verb, conjugations, auxiliary):
                     person = "2"
                 elif part[2] == "P3":
                     person = "3"
+            suffix = suffix_s
+            if auxiliary == "être" and number == "p":
+                suffix = suffix_p
 
             if part[0] == "IPre":
                 conjugated_verb["Indicatif"]["Présent"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Passé composé"][number][person] = aux[auxiliary]["Indicatif"]["Présent"][number][person] + suffix
+                    conjugated_verb["Indicatif"]["Passé composé"][number][person] = \
+                        aux[auxiliary]["Indicatif"]["Présent"][number][person] + suffix
             elif part[0] == "IImp":
                 conjugated_verb["Indicatif"]["Imparfait"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Plus-que-parfait"][number][person] = aux[auxiliary]["Indicatif"]["Imparfait"][number][person] + suffix
+                    conjugated_verb["Indicatif"]["Plus-que-parfait"][number][person] = \
+                        aux[auxiliary]["Indicatif"]["Imparfait"][number][person] + suffix
             elif part[0] == "IPSim":
                 conjugated_verb["Indicatif"]["Passé simple"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Passé antérieur"][number][person] = aux[auxiliary]["Indicatif"]["Passé simple"][number][person] + suffix
+                    conjugated_verb["Indicatif"]["Passé antérieur"][number][person] = \
+                        aux[auxiliary]["Indicatif"]["Passé simple"][number][person] + suffix
             elif part[0] == "IFut":
                 conjugated_verb["Indicatif"]["Futur simple"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Indicatif"]["Futur antérieur"][number][person] = aux[auxiliary]["Indicatif"]["Futur simple"][number][person] + suffix
+                    conjugated_verb["Indicatif"]["Futur antérieur"][number][person] = \
+                        aux[auxiliary]["Indicatif"]["Futur simple"][number][person] + suffix
             elif part[0] == "CPre":
                 conjugated_verb["Conditionnel"]["Présent"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Conditionnel"]["Passé"][number][person] = aux[auxiliary]["Conditionnel"]["Présent"][number][person] + suffix
+                    conjugated_verb["Conditionnel"]["Passé"][number][person] = \
+                        aux[auxiliary]["Conditionnel"]["Présent"][number][person] + suffix
             elif part[0] == "SPre":
                 conjugated_verb["Subjonctif"]["Présent"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Subjonctif"]["Passé"][number][person] = aux[auxiliary]["Subjonctif"]["Présent"][number][person] + suffix
+                    conjugated_verb["Subjonctif"]["Passé"][number][person] = \
+                        aux[auxiliary]["Subjonctif"]["Présent"][number][person] + suffix
             elif part[0] == "SImp":
                 conjugated_verb["Subjonctif"]["Imparfait"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Subjonctif"]["Plus-que-parfait"][number][person] = aux[auxiliary]["Subjonctif"]["Imparfait"][number][person] + suffix
+                    conjugated_verb["Subjonctif"]["Plus-que-parfait"][number][person] = \
+                        aux[auxiliary]["Subjonctif"]["Imparfait"][number][person] + suffix
             elif part[0] == "ImPre":
                 conjugated_verb["Impératif"]["Présent"][number][person] = conjugation
                 if suffix:
-                    conjugated_verb["Impératif"]["Passé"][number][person] = aux[auxiliary]["Impératif"]["Présent"][number][person] + suffix
+                    conjugated_verb["Impératif"]["Passé"][number][person] = \
+                        aux[auxiliary]["Impératif"]["Présent"][number][person] + suffix
             elif part[0] == "PPre":
                 conjugated_verb["Participe"]["Présent"] = conjugation
                 conjugated_verb["Gérondif"]["Présent"] = conjugation
                 if suffix:
-                    conjugated_verb["Participe"]["Passé"]["a"] = aux[auxiliary]["Participe"]["Présent"] + suffix
-                    conjugated_verb["Gérondif"]["Passé"] = conjugated_verb["Participe"]["Passé"]["a"]
+                    conjugated_verb["Participe"]["Passé"]["a"] = \
+                        aux[auxiliary]["Participe"]["Présent"] + suffix
+                    conjugated_verb["Gérondif"]["Passé"] = \
+                        conjugated_verb["Participe"]["Passé"]["a"]
             elif part[0] == "PPas":
                 conjugated_verb["Participe"]["Passé"][number][gender] = conjugation
 
     time_stop = time.time()
-    logging.debug("fill_verb_from_abu_dictionary_data() time: %f / conjugations: %d", time_stop - time_start, len(conjugations))
+    logging.debug(
+        "fill_verb_from_abu_dictionary_data() "
+        + _("time")
+        + ": %f / "
+        + _("conjugations")
+        + ": %d",
+        time_stop - time_start, len(conjugations)
+    )
 
     return conjugated_verb
 
@@ -477,14 +614,54 @@ def conjuguer(verb, conjugations, auxiliary):
     return conjugated_verb
 
 
+################################################################################
+def analyze_verb(verb):
+    """Return a verb pattern, group and conjugation model"""
+    group = None
+    group_text = ""
+    model = ""
+
+    for key in patterns.keys():
+        if verb.endswith(key):
+            group = patterns[key][0]
+            model = patterns[key][1]
+            break
+
+    if group == 0:
+        group_text = _("auxiliary")
+    elif group == 1:
+        group_text = _("1st group")
+    elif group == 2:
+        group_text = _("2nd group")
+    elif group == 3:
+        group_text = _("3rd group")
+    else:
+        group_text = _("unknown group")
+
+    return key, group_text, model
+
+
 ################################################################################
 def print_verb(verb):
     """Return lines describing the conjugated verb"""
+    pattern, group, model = analyze_verb(verb)
     lines = []
     if parameters["Color display"]:
-        lines.append(VERB_COLOR + "Tableau de conjugaison du verbe " + colorama.Style.RESET_ALL + verb)
+        lines.append(
+            VERB_COLOR
+            + _("Conjugation tables for")
+            + " "
+            + colorama.Style.RESET_ALL
+            + verb
+        )
     else:
-        lines.append("Tableau de conjugaison du verbe " + verb)
+        lines.append(_("Conjugation tables for") + " " + verb)
+
+    if verb == model:
+        lines.append(group + ", " + _("model for verbs like") + " *" + pattern)
+    else:
+        lines.append(group + ", " + _("conjugated like") + " " + model)
+
     lines.append("")
 
     return lines
@@ -528,7 +705,10 @@ def get_pronoun(mode, number, person, conjugated_verb):
     elif mode == "Subjonctif":
         if number == "s":
             if person == "1":
-                if conjugated_verb[0] in ['a', 'e', 'i', 'o', 'u', 'y']:
+                if conjugated_verb[0] in [
+                    'a', 'â', 'à', 'ä', 'e', 'ê', 'é', 'è', 'ë',
+                    'i', 'î', 'ï', 'o', 'ô', 'ö', 'u', 'ù', 'y'
+                ]:
                     pronoun = "que j'   "
                 else:
                     pronoun = "que je   "
@@ -546,7 +726,10 @@ def get_pronoun(mode, number, person, conjugated_verb):
     else:
         if number == "s":
             if person == "1":
-                if conjugated_verb[0] in ['a', 'e', 'i', 'o', 'u', 'y']:
+                if conjugated_verb[0] in [
+                    'a', 'â', 'à', 'ä', 'e', 'ê', 'é', 'è', 'ë',
+                    'i', 'î', 'ï', 'o', 'ô', 'ö', 'u', 'ù', 'y'
+                ]:
                     pronoun = "j'   "
                 else:
                     pronoun = "je   "
@@ -643,7 +826,10 @@ def print_verb_conjugation_odd_columns(conjugation):
     column += print_verb(conjugation["Infinitif"]["Présent"])
 
     column += print_mode("Indicatif")
-    for tense in ["Présent", "Imparfait", "Passé simple", "Futur simple", "Passé composé", "Plus-que-parfait", "Passé antérieur", "Futur antérieur"]:
+    for tense in [
+        "Présent", "Imparfait", "Passé simple", "Futur simple",
+        "Passé composé", "Plus-que-parfait", "Passé antérieur", "Futur antérieur"
+    ]:
         column += get_tense_conjugation("Indicatif", tense, conjugation)
 
     column += print_mode("Conditionnel")
@@ -780,23 +966,22 @@ def main():
     """The program's main entry point"""
     program_name = os.path.basename(sys.argv[0])
 
-    console_log_format = program_name + ": %(levelname)s: %(message)s"
-    logging.basicConfig(format=console_log_format, level=logging.DEBUG)
-    logging.disable(logging.INFO)
-
+    initialize_debugging(program_name)
+    initialize_internationalization(program_name)
     process_environment_variables()
-    arguments = process_command_line()
-
-    if parameters["Dictionary type"] not in ("ABU", "DELA"):
-        logging.debug("Unknown inflected dictionary format")
-        sys.exit(1)
-    verbs = load_all_verbs_from_dictionary()
+    arguments = process_command_line(program_name)
 
     if not arguments:
-        logging.critical("conjuguer expects at least one argument")
+        logging.critical(_("conjuguer expects at least one argument"))
         display_help()
         sys.exit(1)
 
+    if parameters["Dictionary type"] not in ("ABU", "DELA"):
+        logging.debug(_("Unknown inflected dictionary format"))
+        sys.exit(1)
+
+    verbs = load_all_verbs_from_dictionary()
+
     exit_status = 0
     for argument in arguments:
         conjugations = select_verb_from_verbs(argument, verbs)
@@ -813,7 +998,17 @@ def main():
                 verb = conjuguer(argument, conjugations, "avoir")
                 print_verb_conjugation(verb)
         else:
-            logging.error("%s is not in the dictionary used", argument)
+            logging.error("%s " + _("is not in the dictionary used"), argument)
+            pattern, group, model = analyze_verb(argument)
+            print(
+                _("If it really exists, it would be")
+                + " "
+                + group
+                + ", "
+                + _("conjugated like")
+                + " "
+                + model
+            )
             exit_status = 1
 
     sys.exit(exit_status)
